@@ -1,7 +1,6 @@
 from __future__ import annotations
 
 import json
-import os
 
 from pathlib import Path
 from typing import Any
@@ -11,7 +10,7 @@ import jsonschema
 from poetry.core.json import SCHEMA_DIR as CORE_SCHEMA_DIR
 
 
-SCHEMA_DIR = os.path.join(os.path.dirname(__file__), "schemas")
+SCHEMA_DIR = Path(__file__).parent / "schemas"
 
 
 class ValidationError(ValueError):
@@ -39,7 +38,7 @@ def validate_object(obj: dict[str, Any]) -> list[str]:
         errors.append(message)
 
     core_schema = json.loads(
-        Path(CORE_SCHEMA_DIR, "poetry-schema.json").read_text(encoding="utf-8")
+        (CORE_SCHEMA_DIR / "poetry-schema.json").read_text(encoding="utf-8")
     )
 
     properties = {*schema["properties"].keys(), *core_schema["properties"].keys()}
