@@ -4,7 +4,6 @@ import csv
 import hashlib
 import json
 import os
-import shutil
 
 from base64 import urlsafe_b64encode
 from pathlib import Path
@@ -44,6 +43,7 @@ WINDOWS_CMD_TEMPLATE = """\
 
 class EditableBuilder(Builder):
     def __init__(self, poetry: Poetry, env: Env, io: IO) -> None:
+        self._poetry: Poetry
         super().__init__(poetry)
 
         self._env = env
@@ -105,19 +105,15 @@ class EditableBuilder(Builder):
                 pip_install(self._path, self._env, upgrade=True, editable=True)
             else:
                 # Temporarily rename pyproject.toml
-                shutil.move(
-                    str(self._poetry.file), str(self._poetry.file.with_suffix(".tmp"))
-                )
+                renamed_pyproject = self._poetry.file.path.with_suffix(".tmp")
+                self._poetry.file.path.rename(renamed_pyproject)
                 try:
                     pip_install(self._path, self._env, upgrade=True, editable=True)
                 finally:
-                    shutil.move(
-                        str(self._poetry.file.with_suffix(".tmp")),
-                        str(self._poetry.file),
-                    )
+                    renamed_pyproject.rename(self._poetry.file.path)
         finally:
             if not has_setup:
-                os.remove(str(setup))
+                os.remove(setup)
 
     def _add_pth(self) -> list[Path]:
         paths = {
@@ -134,7 +130,7 @@ class EditableBuilder(Builder):
         for file in self._env.site_packages.find(path=pth_file, writable_only=True):
             self._debug(
                 f"  - Removing existing <c2>{file.name}</c2> from <b>{file.parent}</b>"
-                f" for {self._poetry.file.parent}"
+                f" for {self._poetry.file.path.parent}"
             )
             # We can't use unlink(missing_ok=True) because it's not always available
             if file.exists():
@@ -146,14 +142,14 @@ class EditableBuilder(Builder):
             )
             self._debug(
                 f"  - Adding <c2>{pth_file.name}</c2> to <b>{pth_file.parent}</b> for"
-                f" {self._poetry.file.parent}"
+                f" {self._poetry.file.path.parent}"
             )
             return [pth_file]
         except OSError:
             # TODO: Replace with PermissionError
             self._io.write_error_line(
                 f"  - Failed to create <c2>{pth_file.name}</c2> for"
-                f" {self._poetry.file.parent}"
+                f" {self._poetry.file.path.parent}"
             )
             return []
 
@@ -167,7 +163,7 @@ class EditableBuilder(Builder):
         else:
             self._io.write_error_line(
                 "  - Failed to find a suitable script installation directory for"
-                f" {self._poetry.file.parent}"
+                f" {self._poetry.file.path.parent}"
             )
             return []
 
@@ -249,7 +245,7 @@ class EditableBuilder(Builder):
             json.dumps(
                 {
                     "dir_info": {"editable": True},
-                    "url": self._poetry.file.path.parent.as_uri(),
+                    "url": self._poetry.file.path.parent.absolute().as_uri(),
                 }
             )
         )
