@@ -1,7 +1,6 @@
 from __future__ import annotations
 
 from typing import TYPE_CHECKING
-from typing import cast
 
 from poetry.core.constraints.version import parse_constraint
 
@@ -114,7 +113,8 @@ class _Writer:
         conjunction = "So," if conclusion or incompatibility == self._root else "And"
         incompatibility_string = str(incompatibility)
 
-        cause: ConflictCause = cast(ConflictCause, incompatibility.cause)
+        cause = incompatibility.cause
+        assert isinstance(cause, ConflictCause)
 
         if isinstance(cause.conflict.cause, ConflictCause) and isinstance(
             cause.other.cause, ConflictCause
@@ -144,8 +144,10 @@ class _Writer:
                 self._visit(without_line)
                 self._write(
                     incompatibility,
-                    f"{conjunction} because {with_line!s} ({line}),"
-                    f" {incompatibility_string}.",
+                    (
+                        f"{conjunction} because {with_line!s} ({line}),"
+                        f" {incompatibility_string}."
+                    ),
                     numbered=numbered,
                 )
             else:
@@ -170,9 +172,11 @@ class _Writer:
 
                     self._write(
                         incompatibility,
-                        f"{conjunction} because"
-                        f" {cause.conflict!s} ({self._line_numbers[cause.conflict]}),"
-                        f" {incompatibility_string}",
+                        (
+                            f"{conjunction} because {cause.conflict!s}"
+                            f" ({self._line_numbers[cause.conflict]}),"
+                            f" {incompatibility_string}"
+                        ),
                         numbered=numbered,
                     )
         elif isinstance(cause.conflict.cause, ConflictCause) or isinstance(
@@ -198,7 +202,8 @@ class _Writer:
                     numbered=numbered,
                 )
             elif self._is_collapsible(derived):
-                derived_cause: ConflictCause = cast(ConflictCause, derived.cause)
+                derived_cause = derived.cause
+                assert isinstance(derived_cause, ConflictCause)
                 if isinstance(derived_cause.conflict.cause, ConflictCause):
                     collapsed_derived = derived_cause.conflict
                     collapsed_ext = derived_cause.other
@@ -233,7 +238,8 @@ class _Writer:
         if self._derivations[incompatibility] > 1:
             return False
 
-        cause: ConflictCause = cast(ConflictCause, incompatibility.cause)
+        cause = incompatibility.cause
+        assert isinstance(cause, ConflictCause)
         if isinstance(cause.conflict.cause, ConflictCause) and isinstance(
             cause.other.cause, ConflictCause
         ):
