@@ -9,10 +9,10 @@ import os
 import platform
 import plistlib
 import re
+import shutil
 import subprocess
 import sys
 import sysconfig
-import warnings
 
 from contextlib import contextmanager
 from copy import deepcopy
@@ -33,10 +33,10 @@ from packaging.tags import interpreter_version
 from packaging.tags import sys_tags
 from poetry.core.constraints.version import Version
 from poetry.core.constraints.version import parse_constraint
-from poetry.core.toml.file import TOMLFile
 from poetry.core.utils.helpers import temporary_directory
 from virtualenv.seed.wheels.embed import get_embed_wheel
 
+from poetry.toml.file import TOMLFile
 from poetry.utils._compat import WINDOWS
 from poetry.utils._compat import decode
 from poetry.utils._compat import encode
@@ -189,33 +189,15 @@ print(json.dumps(sysconfig.get_paths()))
 """
 
 GET_PATHS_FOR_GENERIC_ENVS = """\
-# We can't use sysconfig.get_paths() because
-# on some distributions it does not return the proper paths
-# (those used by pip for instance). We go through distutils
-# to get the proper ones.
 import json
 import site
 import sysconfig
 
-from distutils.command.install import SCHEME_KEYS
-from distutils.core import Distribution
-
-d = Distribution()
-d.parse_config_files()
-obj = d.get_command_obj("install", create=True)
-obj.finalize_options()
-
 paths = sysconfig.get_paths().copy()
-for key in SCHEME_KEYS:
-    if key == "headers":
-        # headers is not a path returned by sysconfig.get_paths()
-        continue
 
-    paths[key] = getattr(obj, f"install_{key}")
-
-if site.check_enableusersite() and hasattr(obj, "install_usersite"):
-    paths["usersite"] = getattr(obj, "install_usersite")
-    paths["userbase"] = getattr(obj, "install_userbase")
+if site.check_enableusersite():
+    paths["usersite"] = site.getusersitepackages()
+    paths["userbase"] = site.getuserbase()
 
 print(json.dumps(paths))
 """
@@ -307,10 +289,7 @@ class SitePackages:
             )
         )
 
-        yield from metadata.PathDistribution.discover(  # type: ignore[no-untyped-call]
-            name=name,
-            path=path,
-        )
+        yield from metadata.PathDistribution.discover(name=name, path=path)
 
     def find_distribution(
         self, name: str, writable_only: bool = False
@@ -328,9 +307,7 @@ class SitePackages:
             files = [] if distribution.files is None else distribution.files
             for file in files:
                 if file.name.endswith(suffix):
-                    yield Path(
-                        distribution.locate_file(file),  # type: ignore[no-untyped-call]
-                    )
+                    yield Path(distribution.locate_file(file))
 
     def find_distribution_files_with_name(
         self, distribution_name: str, name: str, writable_only: bool = False
@@ -341,18 +318,7 @@ class SitePackages:
             files = [] if distribution.files is None else distribution.files
             for file in files:
                 if file.name == name:
-                    yield Path(
-                        distribution.locate_file(file),  # type: ignore[no-untyped-call]
-                    )
-
-    def find_distribution_nspkg_pth_files(
-        self, distribution_name: str, writable_only: bool = False
-    ) -> Iterable[Path]:
-        return self.find_distribution_files_with_suffix(
-            distribution_name=distribution_name,
-            suffix="-nspkg.pth",
-            writable_only=writable_only,
-        )
+                    yield Path(distribution.locate_file(file))
 
     def find_distribution_direct_url_json_files(
         self, distribution_name: str, writable_only: bool = False
@@ -371,16 +337,15 @@ class SitePackages:
         ):
             files = [] if distribution.files is None else distribution.files
             for file in files:
-                path = Path(
-                    distribution.locate_file(file),  # type: ignore[no-untyped-call]
-                )
+                path = Path(distribution.locate_file(file))
+
                 # We can't use unlink(missing_ok=True) because it's not always available
                 if path.exists():
                     path.unlink()
 
             distribution_path: Path = distribution._path  # type: ignore[attr-defined]
             if distribution_path.exists():
-                remove_directory(str(distribution_path), force=True)
+                remove_directory(distribution_path, force=True)
 
             paths.append(distribution_path)
 
@@ -388,16 +353,13 @@ class SitePackages:
 
     def _path_method_wrapper(
         self,
-        path: str | Path,
+        path: Path,
         method: str,
         *args: Any,
         return_first: bool = True,
         writable_only: bool = False,
         **kwargs: Any,
     ) -> tuple[Path, Any] | list[tuple[Path, Any]]:
-        if isinstance(path, str):
-            path = Path(path)
-
         candidates = self.make_candidates(
             path, writable_only=writable_only, strict=True
         )
@@ -419,17 +381,17 @@ class SitePackages:
 
         raise OSError(f"Unable to access any of {paths_csv(candidates)}")
 
-    def write_text(self, path: str | Path, *args: Any, **kwargs: Any) -> Path:
+    def write_text(self, path: Path, *args: Any, **kwargs: Any) -> Path:
         paths = self._path_method_wrapper(path, "write_text", *args, **kwargs)
         assert isinstance(paths, tuple)
         return paths[0]
 
-    def mkdir(self, path: str | Path, *args: Any, **kwargs: Any) -> Path:
+    def mkdir(self, path: Path, *args: Any, **kwargs: Any) -> Path:
         paths = self._path_method_wrapper(path, "mkdir", *args, **kwargs)
         assert isinstance(paths, tuple)
         return paths[0]
 
-    def exists(self, path: str | Path) -> bool:
+    def exists(self, path: Path) -> bool:
         return any(
             value[-1]
             for value in self._path_method_wrapper(path, "exists", return_first=False)
@@ -437,7 +399,7 @@ class SitePackages:
 
     def find(
         self,
-        path: str | Path,
+        path: Path,
         writable_only: bool = False,
     ) -> list[Path]:
         return [
@@ -475,6 +437,11 @@ class EnvCommandError(EnvError):
         super().__init__("\n\n".join(message_parts))
 
 
+class PythonVersionNotFound(EnvError):
+    def __init__(self, expected: str) -> None:
+        super().__init__(f"Could not find the python executable {expected}")
+
+
 class NoCompatiblePythonVersionFound(EnvError):
     def __init__(self, expected: str, given: str | None = None) -> None:
         if given:
@@ -520,34 +487,39 @@ class EnvManager:
         self._io = io or NullIO()
 
     @staticmethod
-    def _full_python_path(python: str) -> str:
+    def _full_python_path(python: str) -> Path | None:
+        # eg first find pythonXY.bat on windows.
+        path_python = shutil.which(python)
+        if path_python is None:
+            return None
+
         try:
             executable = decode(
                 subprocess.check_output(
-                    [python, "-c", "import sys; print(sys.executable)"],
+                    [path_python, "-c", "import sys; print(sys.executable)"],
                 ).strip()
             )
-        except CalledProcessError as e:
-            raise EnvCommandError(e)
+            return Path(executable)
 
-        return executable
+        except CalledProcessError:
+            return None
 
     @staticmethod
-    def _detect_active_python(io: None | IO = None) -> str | None:
+    def _detect_active_python(io: None | IO = None) -> Path | None:
         io = io or NullIO()
-        executable = None
+        io.write_error_line(
+            (
+                "Trying to detect current active python executable as specified in"
+                " the config."
+            ),
+            verbosity=Verbosity.VERBOSE,
+        )
 
-        try:
-            io.write_error_line(
-                (
-                    "Trying to detect current active python executable as specified in"
-                    " the config."
-                ),
-                verbosity=Verbosity.VERBOSE,
-            )
-            executable = EnvManager._full_python_path("python")
+        executable = EnvManager._full_python_path("python")
+
+        if executable is not None:
             io.write_error_line(f"Found: {executable}", verbosity=Verbosity.VERBOSE)
-        except EnvCommandError:
+        else:
             io.write_error_line(
                 (
                     "Unable to detect the current active python executable. Falling"
@@ -555,6 +527,7 @@ class EnvManager:
                 ),
                 verbosity=Verbosity.VERBOSE,
             )
+
         return executable
 
     @staticmethod
@@ -571,7 +544,7 @@ class EnvManager:
             if executable:
                 python_patch = decode(
                     subprocess.check_output(
-                        [executable, "-c", GET_PYTHON_VERSION_ONELINER],
+                        [str(executable), "-c", GET_PYTHON_VERSION_ONELINER],
                     ).strip()
                 )
 
@@ -579,9 +552,14 @@ class EnvManager:
 
         return Version.parse(version)
 
+    @property
+    def in_project_venv(self) -> Path:
+        venv: Path = self._poetry.file.path.parent / ".venv"
+        return venv
+
     def activate(self, python: str) -> Env:
         venv_path = self._poetry.config.virtualenvs_path
-        cwd = self._poetry.file.parent
+        cwd = self._poetry.file.path.parent
 
         envs_file = TOMLFile(venv_path / self.ENVS_FILE)
 
@@ -594,12 +572,14 @@ class EnvManager:
             # Executable in PATH or full executable path
             pass
 
-        python = self._full_python_path(python)
+        python_path = self._full_python_path(python)
+        if python_path is None:
+            raise PythonVersionNotFound(python)
 
         try:
             python_version_string = decode(
                 subprocess.check_output(
-                    [python, "-c", GET_PYTHON_VERSION_ONELINER],
+                    [str(python_path), "-c", GET_PYTHON_VERSION_ONELINER],
                 )
             )
         except CalledProcessError as e:
@@ -610,12 +590,11 @@ class EnvManager:
         patch = python_version.text
 
         create = False
-        is_root_venv = self._poetry.config.get("virtualenvs.in-project")
-        # If we are required to create the virtual environment in the root folder,
+        # If we are required to create the virtual environment in the project directory,
         # create or recreate it if needed
-        if is_root_venv:
+        if self.use_in_project_venv():
             create = False
-            venv = self._poetry.file.parent / ".venv"
+            venv = self.in_project_venv
             if venv.exists():
                 # We need to check if the patch version is correct
                 _venv = VirtualEnv(venv)
@@ -624,7 +603,7 @@ class EnvManager:
                 if patch != current_patch:
                     create = True
 
-            self.create_venv(executable=python, force=create)
+            self.create_venv(executable=python_path, force=create)
 
             return self.get(reload=True)
 
@@ -658,7 +637,7 @@ class EnvManager:
                 if patch != current_patch:
                     create = True
 
-            self.create_venv(executable=python, force=create)
+            self.create_venv(executable=python_path, force=create)
 
         # Activate
         envs[base_env_name] = {"minor": minor, "patch": patch}
@@ -669,7 +648,7 @@ class EnvManager:
     def deactivate(self) -> None:
         venv_path = self._poetry.config.virtualenvs_path
         name = self.generate_env_name(
-            self._poetry.package.name, str(self._poetry.file.parent)
+            self._poetry.package.name, str(self._poetry.file.path.parent)
         )
 
         envs_file = TOMLFile(venv_path / self.ENVS_FILE)
@@ -698,7 +677,7 @@ class EnvManager:
 
         venv_path = self._poetry.config.virtualenvs_path
 
-        cwd = self._poetry.file.parent
+        cwd = self._poetry.file.path.parent
         envs_file = TOMLFile(venv_path / self.ENVS_FILE)
         env = None
         base_env_name = self.generate_env_name(self._poetry.package.name, str(cwd))
@@ -719,12 +698,8 @@ class EnvManager:
 
         if not in_venv or env is not None:
             # Checking if a local virtualenv exists
-            if (
-                self._poetry.config.get("virtualenvs.in-project") is not False
-                and (cwd / ".venv").exists()
-                and (cwd / ".venv").is_dir()
-            ):
-                venv = cwd / ".venv"
+            if self.in_project_venv_exists():
+                venv = self.in_project_venv
 
                 return VirtualEnv(venv)
 
@@ -757,18 +732,12 @@ class EnvManager:
         if name is None:
             name = self._poetry.package.name
 
-        venv_name = self.generate_env_name(name, str(self._poetry.file.parent))
+        venv_name = self.generate_env_name(name, str(self._poetry.file.path.parent))
         venv_path = self._poetry.config.virtualenvs_path
-        env_list = [
-            VirtualEnv(Path(p)) for p in sorted(venv_path.glob(f"{venv_name}-py*"))
-        ]
+        env_list = [VirtualEnv(p) for p in sorted(venv_path.glob(f"{venv_name}-py*"))]
 
-        venv = self._poetry.file.parent / ".venv"
-        if (
-            self._poetry.config.get("virtualenvs.in-project") is not False
-            and venv.exists()
-            and venv.is_dir()
-        ):
+        if self.in_project_venv_exists():
+            venv = self.in_project_venv
             env_list.insert(0, VirtualEnv(venv))
         return env_list
 
@@ -784,7 +753,7 @@ class EnvManager:
     def remove(self, python: str) -> Env:
         venv_path = self._poetry.config.virtualenvs_path
 
-        cwd = self._poetry.file.parent
+        cwd = self._poetry.file.path.parent
         envs_file = TOMLFile(venv_path / self.ENVS_FILE)
         base_env_name = self.generate_env_name(self._poetry.package.name, str(cwd))
 
@@ -837,7 +806,7 @@ class EnvManager:
         else:
             venv_path = self._poetry.config.virtualenvs_path
             # Get all the poetry envs, even for other projects
-            env_names = [Path(p).name for p in sorted(venv_path.glob("*-*-py*"))]
+            env_names = [p.name for p in sorted(venv_path.glob("*-*-py*"))]
             if python in env_names:
                 raise IncorrectEnvError(python)
 
@@ -882,16 +851,30 @@ class EnvManager:
 
         return VirtualEnv(venv_path, venv_path)
 
+    def use_in_project_venv(self) -> bool:
+        in_project: bool | None = self._poetry.config.get("virtualenvs.in-project")
+        if in_project is not None:
+            return in_project
+
+        return self.in_project_venv.is_dir()
+
+    def in_project_venv_exists(self) -> bool:
+        in_project: bool | None = self._poetry.config.get("virtualenvs.in-project")
+        if in_project is False:
+            return False
+
+        return self.in_project_venv.is_dir()
+
     def create_venv(
         self,
         name: str | None = None,
-        executable: str | None = None,
+        executable: Path | None = None,
         force: bool = False,
     ) -> Env:
         if self._env is not None and not force:
             return self._env
 
-        cwd = self._poetry.file.parent
+        cwd = self._poetry.file.path.parent
         env = self.get(reload=True)
 
         if not env.is_sane():
@@ -909,7 +892,7 @@ class EnvManager:
             return env
 
         create_venv = self._poetry.config.get("virtualenvs.create")
-        root_venv = self._poetry.config.get("virtualenvs.in-project")
+        in_project_venv = self.use_in_project_venv()
         prefer_active_python = self._poetry.config.get(
             "virtualenvs.prefer-active-python"
         )
@@ -918,17 +901,20 @@ class EnvManager:
         if not executable and prefer_active_python:
             executable = self._detect_active_python()
 
-        venv_path = cwd / ".venv" if root_venv else self._poetry.config.virtualenvs_path
+        venv_path = (
+            self.in_project_venv
+            if in_project_venv
+            else self._poetry.config.virtualenvs_path
+        )
         if not name:
             name = self._poetry.package.name
-        assert name is not None
 
         python_patch = ".".join([str(v) for v in sys.version_info[:3]])
         python_minor = ".".join([str(v) for v in sys.version_info[:2]])
         if executable:
             python_patch = decode(
                 subprocess.check_output(
-                    [executable, "-c", GET_PYTHON_VERSION_ONELINER],
+                    [str(executable), "-c", GET_PYTHON_VERSION_ONELINER],
                 ).strip()
             )
             python_minor = ".".join(python_patch.split(".")[:2])
@@ -952,44 +938,42 @@ class EnvManager:
                 "Trying to find and use a compatible version.</warning> "
             )
 
-            for python_to_try in sorted(
+            for suffix in sorted(
                 self._poetry.package.AVAILABLE_PYTHONS,
                 key=lambda v: (v.startswith("3"), -len(v), v),
                 reverse=True,
             ):
-                if len(python_to_try) == 1:
-                    if not parse_constraint(f"^{python_to_try}.0").allows_any(
+                if len(suffix) == 1:
+                    if not parse_constraint(f"^{suffix}.0").allows_any(
                         supported_python
                     ):
                         continue
-                elif not supported_python.allows_any(
-                    parse_constraint(python_to_try + ".*")
-                ):
+                elif not supported_python.allows_any(parse_constraint(suffix + ".*")):
                     continue
 
-                python = "python" + python_to_try
-
+                python_name = f"python{suffix}"
                 if self._io.is_debug():
-                    self._io.write_error_line(f"<debug>Trying {python}</debug>")
+                    self._io.write_error_line(f"<debug>Trying {python_name}</debug>")
+
+                python = self._full_python_path(python_name)
+                if python is None:
+                    continue
 
                 try:
                     python_patch = decode(
                         subprocess.check_output(
-                            [python, "-c", GET_PYTHON_VERSION_ONELINER],
+                            [str(python), "-c", GET_PYTHON_VERSION_ONELINER],
                             stderr=subprocess.STDOUT,
                         ).strip()
                     )
                 except CalledProcessError:
                     continue
 
-                if not python_patch:
-                    continue
-
                 if supported_python.allows(Version.parse(python_patch)):
                     self._io.write_error_line(
-                        f"Using <c1>{python}</c1> ({python_patch})"
+                        f"Using <c1>{python_name}</c1> ({python_patch})"
                     )
-                    executable = self._full_python_path(python)
+                    executable = python
                     python_minor = ".".join(python_patch.split(".")[:2])
                     break
 
@@ -998,7 +982,7 @@ class EnvManager:
                     self._poetry.package.python_versions
                 )
 
-        if root_venv:
+        if in_project_venv:
             venv = venv_path
         else:
             name = self.generate_env_name(name, str(cwd))
@@ -1071,8 +1055,8 @@ class EnvManager:
     @classmethod
     def build_venv(
         cls,
-        path: Path | str,
-        executable: str | Path | None = None,
+        path: Path,
+        executable: Path | None = None,
         flags: dict[str, bool] | None = None,
         with_pip: bool | None = None,
         with_wheel: bool | None = None,
@@ -1103,14 +1087,13 @@ class EnvManager:
             else flags.pop("no-wheel", flags["no-pip"])
         )
 
-        if isinstance(executable, Path):
-            executable = executable.resolve().as_posix()
+        executable_str = None if executable is None else executable.resolve().as_posix()
 
         args = [
             "--no-download",
             "--no-periodic-update",
             "--python",
-            executable or sys.executable,
+            executable_str or sys.executable,
         ]
 
         if prompt is not None:
@@ -1138,9 +1121,7 @@ class EnvManager:
         return cli_result
 
     @classmethod
-    def remove_venv(cls, path: Path | str) -> None:
-        if isinstance(path, str):
-            path = Path(path)
+    def remove_venv(cls, path: Path) -> None:
         assert path.is_dir()
         try:
             remove_directory(path)
@@ -1228,10 +1209,7 @@ class Env:
             path = get_real_windows_path(path)
             base = get_real_windows_path(base) if base else None
 
-        if not self._is_windows or self._is_mingw:
-            bin_dir = "bin"
-        else:
-            bin_dir = "Scripts"
+        bin_dir = "bin" if not self._is_windows or self._is_mingw else "Scripts"
         self._path = path
         self._bin_dir = self._path / bin_dir
 
@@ -1251,7 +1229,7 @@ class Env:
         self._platlib: Path | None = None
         self._script_dirs: list[Path] | None = None
 
-        self._embedded_pip_path: str | None = None
+        self._embedded_pip_path: Path | None = None
 
     @property
     def path(self) -> Path:
@@ -1262,8 +1240,9 @@ class Env:
         return self._base
 
     @property
-    def version_info(self) -> tuple[Any, ...]:
-        return tuple(self.marker_env["version_info"])
+    def version_info(self) -> tuple[int, int, int, str, int]:
+        version_info: tuple[int, int, int, str, int] = self.marker_env["version_info"]
+        return version_info
 
     @property
     def python_implementation(self) -> str:
@@ -1271,11 +1250,11 @@ class Env:
         return implementation
 
     @property
-    def python(self) -> str:
+    def python(self) -> Path:
         """
         Path to current python executable
         """
-        return self._bin(self._executable)
+        return Path(self._bin(self._executable))
 
     @property
     def marker_env(self) -> dict[str, Any]:
@@ -1331,21 +1310,21 @@ class Env:
         return path
 
     @property
-    def pip_embedded(self) -> str:
+    def pip_embedded(self) -> Path:
         if self._embedded_pip_path is None:
-            self._embedded_pip_path = str(self.get_embedded_wheel("pip") / "pip")
+            self._embedded_pip_path = self.get_embedded_wheel("pip") / "pip"
         return self._embedded_pip_path
 
     @property
-    def pip(self) -> str:
+    def pip(self) -> Path:
         """
         Path to current pip executable
         """
         # we do not use as_posix() here due to issues with windows pathlib2
         # implementation
-        path = self._bin(self._pip_executable)
-        if not Path(path).exists():
-            return str(self.pip_embedded)
+        path = Path(self._bin(self._pip_executable))
+        if not path.exists():
+            return self.pip_embedded
         return path
 
     @property
@@ -1464,9 +1443,9 @@ class Env:
 
     def get_pip_command(self, embedded: bool = False) -> list[str]:
         if embedded or not Path(self._bin(self._pip_executable)).exists():
-            return [self.python, self.pip_embedded]
+            return [str(self.python), str(self.pip_embedded)]
         # run as module so that pip can update itself on Windows
-        return [self.python, "-m", "pip"]
+        return [str(self.python), "-m", "pip"]
 
     def get_supported_tags(self) -> list[Tag]:
         raise NotImplementedError()
@@ -1495,16 +1474,16 @@ class Env:
 
         return [self._bin(bin)]
 
-    def run(self, bin: str, *args: str, **kwargs: Any) -> str | int:
+    def run(self, bin: str, *args: str, **kwargs: Any) -> str:
         cmd = self.get_command_from_bin(bin) + list(args)
         return self._run(cmd, **kwargs)
 
-    def run_pip(self, *args: str, **kwargs: Any) -> int | str:
+    def run_pip(self, *args: str, **kwargs: Any) -> str:
         pip = self.get_pip_command()
         cmd = pip + list(args)
         return self._run(cmd, **kwargs)
 
-    def run_python_script(self, content: str, **kwargs: Any) -> int | str:
+    def run_python_script(self, content: str, **kwargs: Any) -> str:
         return self.run(
             self._executable,
             "-I",
@@ -1516,7 +1495,7 @@ class Env:
             **kwargs,
         )
 
-    def _run(self, cmd: list[str], **kwargs: Any) -> int | str:
+    def _run(self, cmd: list[str], **kwargs: Any) -> str:
         """
         Run a command inside the Python environment.
         """
@@ -1538,7 +1517,8 @@ class Env:
                 ).stdout
             elif call:
                 assert stderr != subprocess.PIPE
-                return subprocess.call(cmd, stderr=stderr, env=env, **kwargs)
+                subprocess.check_call(cmd, stderr=stderr, env=env, **kwargs)
+                output = ""
             else:
                 output = subprocess.check_output(cmd, stderr=stderr, env=env, **kwargs)
         except CalledProcessError as e:
@@ -1615,8 +1595,8 @@ class SystemEnv(Env):
     """
 
     @property
-    def python(self) -> str:
-        return sys.executable
+    def python(self) -> Path:
+        return Path(sys.executable)
 
     @property
     def sys_path(self) -> list[str]:
@@ -1629,37 +1609,13 @@ class SystemEnv(Env):
         return platform.python_implementation()
 
     def get_paths(self) -> dict[str, str]:
-        # We can't use sysconfig.get_paths() because
-        # on some distributions it does not return the proper paths
-        # (those used by pip for instance). We go through distutils
-        # to get the proper ones.
         import site
 
-        from distutils.command.install import SCHEME_KEYS
-        from distutils.core import Distribution
-
-        d = Distribution()
-        d.parse_config_files()
-        with warnings.catch_warnings():
-            warnings.filterwarnings("ignore", "setup.py install is deprecated")
-            obj = d.get_command_obj("install", create=True)
-        assert obj is not None
-        obj.finalize_options()
-
         paths = sysconfig.get_paths().copy()
-        for key in SCHEME_KEYS:
-            if key == "headers":
-                # headers is not a path returned by sysconfig.get_paths()
-                continue
-
-            paths[key] = getattr(obj, f"install_{key}")
 
         if site.check_enableusersite():
-            usersite = getattr(obj, "install_usersite", None)
-            userbase = getattr(obj, "install_userbase", None)
-            if usersite is not None and userbase is not None:
-                paths["usersite"] = usersite
-                paths["userbase"] = userbase
+            paths["usersite"] = site.getusersitepackages()
+            paths["userbase"] = site.getuserbase()
 
         return paths
 
@@ -1719,13 +1675,11 @@ class VirtualEnv(Env):
         # from inside the virtualenv.
         if base is None:
             output = self.run_python_script(GET_BASE_PREFIX)
-            assert isinstance(output, str)
             self._base = Path(output.strip())
 
     @property
     def sys_path(self) -> list[str]:
         output = self.run_python_script(GET_SYS_PATH)
-        assert isinstance(output, str)
         paths: list[str] = json.loads(output)
         return paths
 
@@ -1741,20 +1695,17 @@ class VirtualEnv(Env):
 
     def get_supported_tags(self) -> list[Tag]:
         output = self.run_python_script(GET_SYS_TAGS)
-        assert isinstance(output, str)
 
         return [Tag(*t) for t in json.loads(output)]
 
     def get_marker_env(self) -> dict[str, Any]:
         output = self.run_python_script(GET_ENVIRONMENT_INFO)
-        assert isinstance(output, str)
 
         env: dict[str, Any] = json.loads(output)
         return env
 
     def get_pip_version(self) -> Version:
         output = self.run_pip("--version")
-        assert isinstance(output, str)
         output = output.strip()
 
         m = re.match("pip (.+?)(?: from .+)?$", output)
@@ -1765,7 +1716,6 @@ class VirtualEnv(Env):
 
     def get_paths(self) -> dict[str, str]:
         output = self.run_python_script(GET_PATHS)
-        assert isinstance(output, str)
         paths: dict[str, str] = json.loads(output)
         return paths
 
@@ -1776,7 +1726,7 @@ class VirtualEnv(Env):
         # A virtualenv is considered sane if "python" exists.
         return os.path.exists(self.python)
 
-    def _run(self, cmd: list[str], **kwargs: Any) -> int | str:
+    def _run(self, cmd: list[str], **kwargs: Any) -> str:
         kwargs["env"] = self.get_temp_environ(environ=kwargs.get("env"))
         return super()._run(cmd, **kwargs)
 
@@ -1881,7 +1831,6 @@ class GenericEnv(VirtualEnv):
 
     def get_paths(self) -> dict[str, str]:
         output = self.run_python_script(GET_PATHS_FOR_GENERIC_ENVS)
-        assert isinstance(output, str)
 
         paths: dict[str, str] = json.loads(output)
         return paths
@@ -1898,7 +1847,7 @@ class GenericEnv(VirtualEnv):
 
         return exe.returncode
 
-    def _run(self, cmd: list[str], **kwargs: Any) -> int | str:
+    def _run(self, cmd: list[str], **kwargs: Any) -> str:
         return super(VirtualEnv, self)._run(cmd, **kwargs)
 
     def is_venv(self) -> bool:
@@ -1928,15 +1877,15 @@ class NullEnv(SystemEnv):
 
         return self._paths
 
-    def _run(self, cmd: list[str], **kwargs: Any) -> int | str:
+    def _run(self, cmd: list[str], **kwargs: Any) -> str:
         self.executed.append(cmd)
 
         if self._execute:
             return super()._run(cmd, **kwargs)
-        return 0
+        return ""
 
     def execute(self, bin: str, *args: str, **kwargs: Any) -> int:
-        self.executed.append([bin] + list(args))
+        self.executed.append([bin, *list(args)])
 
         if self._execute:
             return super().execute(bin, *args, **kwargs)
@@ -1948,14 +1897,14 @@ class NullEnv(SystemEnv):
 
 @contextmanager
 def ephemeral_environment(
-    executable: str | Path | None = None,
+    executable: Path | None = None,
     flags: dict[str, bool] | None = None,
 ) -> Iterator[VirtualEnv]:
     with temporary_directory() as tmp_dir:
         # TODO: cache PEP 517 build environment corresponding to each project venv
         venv_dir = Path(tmp_dir) / ".venv"
         EnvManager.build_venv(
-            path=venv_dir.as_posix(),
+            path=venv_dir,
             executable=executable,
             flags=flags,
         )
